#include "StdAfx.h"
#include "WASGetCallHistory.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"


CWASGetCallHistory::CWASGetCallHistory(CWASActor *pActor) : 
	CWASCommand(pActor)
{
    m_eCallType = callTypeSend;
}


CWASGetCallHistory::~CWASGetCallHistory(void)
{
}


void CWASGetCallHistory::ExecuteAsync()
{
	m_listCallHistory.clear();

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if (pMessengerActor == NULL)
	{
		return;
	}	

	USES_CONVERSION;
	USES_UTF8_CONVERSION;

	WSTRING strURL = _T("");

	strURL = m_pActor->GetWASServerAddress();
	strURL += _T("/ne/call/GetCallHistory.php");

	m_httpFacade.session().SetURL(T2CA(strURL.c_str()), pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);

	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));
	m_httpFacade.session().AddBodyVariable("ticket", pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().AddBodyVariable("langcode", W2U8(m_strLangCode.c_str()));

	char szTemp[10];
	sprintf(szTemp, "%d", (int)m_eCallType);
	m_httpFacade.session().AddBodyVariable("calltype", szTemp);

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}

bool CWASGetCallHistory::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	if(pDom == NULL)
		return false;

	nfc::IIkXMLNode* pNode = pDom->GetdocumentElement()->selectSingleNode("calls");
	if (pNode == NULL)
	{
		m_strWasErrMsg += _T("Call history  \"<calls>\" ã  ϴ.");
		return false;
	}

	nfc::IIkXMLNodeList *pList = pNode->selectNodes("call");	
	nfc::IIkXMLAttribute* pAttr;
	USES_CONVERSION;
	USES_UTF8_CONVERSION;

	m_listCallHistory.clear();

	for (int i = 0; i < pList->Getlength(); i++)
	{	
		NE_CALL_DATA callData;
		memset(&callData, 0x00, sizeof(NE_CALL_DATA));

		pNode = pList->Getitem(i);
		GT_ASSERT(pNode);

		if (NULL == (pAttr = pNode->getAttribute("call_seq")))
			continue;	// ignore this item

		if (NULL != (pAttr = pNode->getAttribute("call_seq")))
			callData.nCallSeq = atoi(pAttr->Gettext().c_str());
		
		if (NULL != (pAttr = pNode->getAttribute("send_user_id")))
			callData.strSendUserId = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("send_emp_nm")))
			callData.strSendUserNm = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("send_dept_nm")))
			callData.strSendDeptNm = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("send_jbps_nm")))
			callData.strSendJbpsNm = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("send_jbdt_nm")))
			callData.strSendJbdtNm = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("send_inco_tel_no")))
			callData.strSendExtensionTelNo = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("send_tel_no")))
			callData.strSendTelNo = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("recv_user_id")))
			callData.strRecvUserId = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("recv_emp_nm")))
			callData.strRecvUserNm = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("recv_dept_nm")))
			callData.strRecvDeptNm = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("recv_jbps_nm")))
			callData.strRecvJbpsNm = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("recv_jbdt_nm")))
			callData.strRecvJbdtNm = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("recv_inco_tel_no")))
			callData.strRecvExtensionTelNo = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("recv_tel_no")))
			callData.strRecvTelNo = U82W(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("start_ymdt")))
			callData.nStartYmdt = atoi(pAttr->Gettext().c_str());

		if (NULL != (pAttr = pNode->getAttribute("end_ymdt")))
			callData.nEndYmdt = atoi(pAttr->Gettext().c_str());

		m_listCallHistory.push_back(callData);
	}

	return true;
}


// override in drived class
void CWASGetCallHistory::OnWasResponse(trid_t trid,
								  bool bSuccess,
								  int /*nWinINetErrorCode*/, 
								  int /*nHttpResponseCode*/, 
                                  LPCSTR /*szWasResCodeInHeader*/, 
                                  LPCSTR szWasResCodeInXml,
                                  nfc::CIkXMLDom * pDom)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(pDom) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		pObserver->OnGTWASActor_ResponseCallHistory(m_pActor, GetID(), m_eError, m_listCallHistory);
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASGetCallHistory::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		pObserver->OnGTWASActor_ResponseCallHistory(m_pActor, GetID(), m_eError, m_listCallHistory);
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}



//// CWASSetCallInfo //////////////////////////////////////////////////////////

CWASSetCallInfo::CWASSetCallInfo(CWASActor *pActor) : 
CWASCommand(pActor)
{
	m_nCallSeq = 0;
}


CWASSetCallInfo::~CWASSetCallInfo(void)
{
}


void CWASSetCallInfo::ExecuteAsync()
{
	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if (pMessengerActor == NULL)
	{
		return;
	}	

	USES_CONVERSION;
	USES_UTF8_CONVERSION;

	WSTRING strURL = _T("");

	strURL = m_pActor->GetWASServerAddress();
	strURL += _T("/ne/call/SetCallInfo.php");

	m_httpFacade.session().SetURL(T2CA(strURL.c_str()), pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);

	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));
	m_httpFacade.session().AddBodyVariable("ticket", pMessengerActor->GetMessengerTicket());

	char szTempCallSeq[32];
	memset(szTempCallSeq, NULL, sizeof(szTempCallSeq));
	sprintf(szTempCallSeq, "%d", (int)m_sCallData.nCallSeq);
	m_httpFacade.session().AddBodyVariable("call_seq", szTempCallSeq);

	m_httpFacade.session().AddBodyVariable("send_user_id", W2U8(m_sCallData.strSendUserId.c_str()));
	m_httpFacade.session().AddBodyVariable("send_tel_no", W2U8(m_sCallData.strSendTelNo.c_str()));
	m_httpFacade.session().AddBodyVariable("recv_user_id", W2U8(m_sCallData.strRecvUserId.c_str()));
	m_httpFacade.session().AddBodyVariable("recv_tel_no", W2U8(m_sCallData.strRecvTelNo.c_str()));

	char szTempStartYmdt[32];
	memset(szTempStartYmdt, NULL, sizeof(szTempStartYmdt));
	sprintf(szTempStartYmdt, "%d", (int)m_sCallData.nStartYmdt);
	m_httpFacade.session().AddBodyVariable("start_ymdt", szTempStartYmdt);

	char szTempEndYmdt[32];
	memset(szTempEndYmdt, NULL, sizeof(szTempEndYmdt));
	sprintf(szTempEndYmdt, "%d", (int)m_sCallData.nEndYmdt);
	m_httpFacade.session().AddBodyVariable("end_ymdt", szTempEndYmdt);

	//m_httpFacade.session().AddBodyVariable("calltype", szTemp);

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}

bool CWASSetCallInfo::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	if(pDom == NULL)
		return false;

	nfc::IIkXMLAttribute * pAttr;
	nfc::IIkXMLNode * pNode;

	pNode = pDom->GetdocumentElement()->selectSingleNode("result");
	if(pNode == NULL)
		return false;

	pAttr = pNode->getAttribute("code");
	if( pAttr )
	{
		int nResult = atoi(pAttr->Gettext().c_str());
		if( nResult == 100 )
		{
			m_eError = wasError_None;

			pNode = pDom->GetdocumentElement()->selectSingleNode("call");
			if (NULL != (pAttr = pNode->getAttribute("call_seq")))
				m_nCallSeq = atoi(pAttr->Gettext().c_str());
		}
		else
			m_eError = wasError_Etc;
	}

	return true;
}


// override in drived class
void CWASSetCallInfo::OnWasResponse(trid_t trid,
									   bool bSuccess,
									   int /*nWinINetErrorCode*/, 
									   int /*nHttpResponseCode*/, 
									   LPCSTR /*szWasResCodeInHeader*/, 
									   LPCSTR szWasResCodeInXml,
									   nfc::CIkXMLDom * pDom)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(pDom) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		pObserver->OnGTWASActor_ResponseSetCallInfo(m_pActor, GetID(), m_eError, m_nCallSeq);
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASSetCallInfo::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		pObserver->OnGTWASActor_ResponseSetCallInfo(m_pActor, GetID(), m_eError, m_nCallSeq);
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}
